#include "btree.h"

int memcount = 0;

void * d_malloc( size_t size )
   {
      ++memcount;
      return malloc(size);
   }

void d_free(void * memblock )
   {
      --memcount;
      free(memblock);
   }

int getmemcount(void)
   {
      return memcount;
   }

/******************************************************************************
 * Wandelt einen String in Kleinbuchstaben
 * 
 * -> s        - Zeiger auf String
 *****************************************************************************/
void cstring_tolower(char *s)
{
 while(*s)
   {
     *s = tolower(*s); 
     s++;
   }
}


/******************************************************************************
 * Liefert einen leeren Baum zurueck
 ******************************************************************************/
Treenode* mkEmpty(void)
{
 return NULL;
}


/******************************************************************************
  Verhaelt sich wie _isEmpty, der Parameter w wird vernachlaessig.
  Grund: Funktion soll mit prad() aufgerufen werden koennen.
  @param Treenode* : Baum der auf Leere geprueft wird.
  return: 1, wenn leer sonst 0.
 ******************************************************************************/
int isEmpty(char* w, Treenode* ptr)
{
 return (ptr==NULL) ? 1 : 0 ;
}


/******************************************************************************
 * Vergleicht zwei Strings und gibt den Unterschied zurck
 ******************************************************************************/
int compare(char *s1, char *s2)
{
 int res;
 char *p1, *p2;
 p1 = d_malloc(strlen(s1)+1);
 strcpy(p1,s1);
 p2 = d_malloc(strlen(s2)+1);
 strcpy(p2,s2);
 
 cstring_tolower(p1);
 cstring_tolower(p2);
 res = strcmp(p1,p2);
 d_free(p1);
 d_free(p2);

 return res;
}


/******************************************************************************
 * vergleicht zwei Strings und gibt zurck, ob diese gleich sind
 ******************************************************************************/
int isEqual(char *s1, char *s2)
{
 return !compare(s1,s2);
}


/******************************************************************************
  prueft, ob w im Baum enthalten ist
  @param char*: Wort, deren Inhalt im Baum gesucht wird.
  @param Treenode*: Baum, in welchem gesucht wird.
  return: 1, wenn das Wort im Baum vorhanden ist, sonst 0.
 ******************************************************************************/
int isIn(char* w, Treenode* ptr)
{
 if (isEmpty(NULL, ptr) || !w)
   return 0;

 if (isEqual(w, ptr->word) )
   return 1;

 return  isIn(w, ptr->left) || isIn(w, ptr->right);
}


/******************************************************************************
  Erzeugt einen neuen Baum mit der Wurzel w
  @param char*: Das Wort, mit dem der Baum erzeugt wird.
  return: der neue Baum.
 ******************************************************************************/
Treenode* mkOne(char* w)
{
 Treenode *t; 

 if (!w)                                  /* kein Wort angegeben */
   return mkEmpty();

 t = d_malloc(sizeof(Treenode));
 if (!t)
   {
      printf("\nmkOne: malloc failed\n");
      exit(1);

   }

 t->left    = mkEmpty();
 t->right   = mkEmpty();

 if (w)                                   /* ist der String gltig? */
   {
      t->word = d_malloc(strlen(w)+1);
      strcpy(t->word, w);
   }
 else
   {
      t->word = NULL;
   }

 return t;
}


/******************************************************************************
 * sortiert einen Baum nach Rechts
 ******************************************************************************/
Treenode * sortRight(Treenode *ptr, Treenode *root)
{

 if (isEmpty(NULL, ptr) )                 /* wenn Baum leer -> Abbruch */          
   return ptr;

 if ( !isEmpty(NULL, ptr->left) )         /* linken Baum sortieren und einfgen */
   root = sortRight(ptr->left, root);

 root = append(ptr->word, root);          /* Wurzel einfgen */

 if ( !isEmpty(NULL, ptr->right) )        /* rechten Baum sortieren und einfgen */
   root = sortRight(ptr->right, root);

 if (ptr->word)                           /* Wurzel freigeben */
   d_free(ptr->word);
 d_free(ptr);
 
 return root;                             /* neue Wurzel zurckgeben */
}


/******************************************************************************
  Produziert einen Baum, welcher keine linken Teilbaeume bestitzt.
  Der Kopf ist das kleinste Element.
  @param Treenode*: Der Baum, welcher sortiert werden soll.
  return Treenode*: Ein sortierter Baum ohne linke Teilbaeume.
 ******************************************************************************/
Treenode* sortToRight(Treenode* ptr)
{
 return sortRight(ptr, mkEmpty() );
}


/******************************************************************************
 * sortiert einen Baum nach Links
 ******************************************************************************/
Treenode * sortLeft(Treenode *ptr, Treenode *root)
{

 if (isEmpty(NULL, ptr) )                 /* wenn Baum leer -> Abbruch */          
   return ptr;

 if ( !isEmpty(NULL, ptr->right) )        /* rechten Baum sortieren und einfgen */
   root = sortLeft(ptr->right, root);

 root = append(ptr->word, root);          /* Wurzel einfgen */

 if ( !isEmpty(NULL, ptr->left) )         /* linken Baum sortieren und einfgen */
   root = sortLeft(ptr->left, root);

 if (ptr->word)                           /* Wurzel freigeben */
   d_free(ptr->word);
 d_free(ptr);
 
 return root;                             /* neue Wurzel zurckgeben */
}


/******************************************************************************
  Produziert einen Baum, welcher keine rechten Teilbaeume bestitzt.
  Der Kopf ist das groesste Element.
  @param Treenode*: Der Baum, welcher sortiert werden soll.
  return Treenode*: Ein sortierter Baum ohne rechte Teilbaeume.
 
 ******************************************************************************/
Treenode* sortToLeft(Treenode* ptr)
{
 return sortLeft(ptr, mkEmpty() );
}


Treenode* balance(Treenode *s, unsigned len)
{
 unsigned   rootPos;
 Treenode   *root;

 if (len==0)
   return mkEmpty();

 rootPos = len/2;
 root = s;
 while (rootPos--)
   root = root->right;
 
 root->left    = balance(s, len/2);
 root->right   = balance(root->right, len- len/2 -1);
 
 return root;
}

/******************************************************************************
  Balanciert den Baum aus.
  Es wird u.U. nicht immer ganz aufgehen.
  @param Treenode*: Der Baum, der ausbalanciert werden soll.
  return Treenode*: Der ausbalancierte Baum.
 ******************************************************************************/
Treenode* sortToBalance(Treenode* ptr)
{
 unsigned len = card(NULL, ptr);
 return balance(sortToRight(ptr), len);
}


/******************************************************************************
  Liefert die Anzahl der Elemente im Baum.
  w verhaelt sich als dummy.
  Grund: Funktion soll mit prad() aufgerufen werden koennen.
  @param Treenode*: Baum, von dem die Anzahl Elemente bestimmt
                    werden soll. Ist der Baum leer, so sind
                    0 Elemente im Baum!
 ******************************************************************************/
int card(char* w, Treenode* ptr)
{
 if (isEmpty(w, ptr) )
   return 0;
 
 return card(w, ptr->left) + 1 + card(w, ptr->right);
}


/******************************************************************************
  Liefert die Ebene, auf welcher sich w im Baum befindet.
  Der Wurzelknoten befindet sich auf 1.
  @param char*: Wort dessen Tiefe im Baum gesucht ist.
  @param Treenode*: Baum in dem w gesucht wird.
  return: Ebene, auf welcher sich w befindet.
  z.B.	1: w ist Wurzelknoten.
	0: w ist nicht im Baum.
 ******************************************************************************/
int ord(char* w, Treenode* ptr)
{
 int i;

 if (isEmpty(w, ptr) || !w)               /* Wort ist nicht im Baum */
   return 0;

 if (isEqual(ptr->word, w) )              /* aktuelles Element ist Wort */
   return 1;
 
 i=ord(w, ptr->left);                     /* wort ist im linken teilbaum */
 if (i > 0)
   return i+1;

 i=ord(w, ptr->right);                    /* wort ist im rechten teilbaum */
 if (i > 0)
   return i+1;
 
 return 0;                                /* wort wurde nicht gefunden */
}


/******************************************************************************
  Haengt w in den bestehenden Baum ein.
  @param char*: Wort, welches in den Baum gehaengt werden soll.
                Ist das Wort schon vorhanden wird es nicht erneut
		eingefuegt und der bestehende Baum unveraendert 
		zurueckgeliefert.
  @param Treenode*: Der Baum, welcher um w erweitert wird.
                Ist der Baum leer, so wird ein neuer Baum mit w
		erzeugt.
  return Treenode*: der neue Baum.
 ******************************************************************************/
Treenode* append(char* w, Treenode* ptr)
{
 int res;

 if (isEmpty(w, ptr) )                    /* noch kein baum da */
   return mkOne(w);

 res = compare(w, ptr->word);

 if (res==0)                              /* wort ist schon im baum */
   return ptr;

 if (res>0)                               /* wort ist grer->rechts einfgen */
   ptr->right=append(w, ptr->right);

 if (res<0)                               /* wort ist kleiner->links einfgen */
   ptr->left=append(w, ptr->left);

 return ptr;
}


/******************************************************************************
 * gibt i Spaces aus
 ******************************************************************************/
void spaces(int i)
{
 while (i--)
   printf(" ");
}


/******************************************************************************
 * zeigt einen Tree an
 ******************************************************************************/
void showtree(Treenode * t, int s)
{
 spaces(s);
 if (!t)
   {
      printf("|--NULL\n");
      return;
   }
 printf("|--%s\n", t->word);

 showtree(t->left,  s+4);
 showtree(t->right, s+4);
}


/******************************************************************************
   Produziert eine Projektion des Baumes auf die Standardausgabe.
   Hierbei soll die Baumstruktur erkennbar sein.
   @param Treenode*: Der Baum, der ausgegeben werden soll
 ******************************************************************************/
void output(Treenode* ptr)
{
 showtree(ptr, 0); 
}


/******************************************************************************
 * ermittelt das grte Element in einem Baum und liefert einen Zeiger auf 
 * dessen String zurck
 ******************************************************************************/
char * tmaxElement(Treenode *s)
{
 if (isEmpty(NULL, s->right) )
   return s->word;

 return tmaxElement(s->right);
}


/******************************************************************************
 * lscht eine Wurzel und sortiert die Anhaengsel neu
 ******************************************************************************/
Treenode* tremoveRoot(Treenode *s)
{
 if (isEmpty(NULL, s->left) || isEmpty(NULL, s->right) )             /* ein Zweig ist leer -> Weiterleitung */
   {
      Treenode *s1 = isEmpty(NULL, s->left) ? s->right : s->left;    /* s1 bekommt Wert des evtl. nicht leeren Zweigs */
      if (s->word)                                                   /* lschen des Elements */
         d_free(s->word);
      d_free(s);
      return s1;                                                     /* Referenz oder NULL zurckgeben */
   }
 else                                                                /* beide Unter-Zweige sind voll */
   {
      char *maxE = tmaxElement(s->left);                             /* suche groetes Element des linken Baumes */
      char *tmp  = d_malloc(strlen(maxE)+1);                         
      strcpy(tmp,maxE);                                              /* merke String des gr. El. des linken Baumes */
      if (s->word)                                                   /* lsche String des zu lschenden El. */
         d_free(s->word);
      s->left = removeTree(maxE, s->left);                              
      s->word = tmp;                                                 /* lschendes El. bekommt Wert d. gr. El. d. linken Baumes */
      return s;                                                      
   }   

}


/******************************************************************************
  Entfernt w aus dem Baum, wenn w vorhanden ist :-)
  Ist der Baum nach dem entfernen von w leer, so ist ein leerer Baum
  zurueckzugeben. 
  @param char*: Wort, welches aus dem Baum entfernt wird. Ist das Wort
                nicht vorhanden, so wird der Baum unverandert zurueckgeliefert.
  @param Treenode*: Der Baum. War der Baum schon vor dem Aufruf leer, so
                wird auch ein leerer Baum zurueckgeliefert.
 ******************************************************************************/
Treenode* removeTree(char* w, Treenode* ptr)
{
 int i;

 if (isEmpty(NULL, ptr) )
   return ptr;

 i = compare(w, ptr->word);

 if (i<0)                                 /* s ist links */
   {
      ptr->left = removeTree(w, ptr->left);
      return ptr;
   }

 if (i>0)                                 /* s ist rechts */
   {
      ptr->right = removeTree(w, ptr->right);
      return ptr;      
   }
 
 return tremoveRoot(ptr);                   /* zu lschendes Element gefunden */

}


/******************************************************************************
  Entfernt w und alles was dran haengt aus dem Baum,
  wenn w vorhanden ist :-)
  @param char*: Wort, welches aus dem Baum geloescht wird.
                Geloescht wird ebenfalls der Baum, welcher an w 
		dranhaengt.
		Ist w nicht vorhanden, so bleibt der Baum unveraendert.
  @param Treenode*: Der Baum. Ist der Baum leer, so ist das
                    Ergebnis ebenfalls ein leerer Baum.
  return Treenode*: Der resultierende Baum
 ******************************************************************************/
Treenode* removePartTree(char* w, Treenode* ptr)
{
 if (isEmpty(NULL, ptr) )                 /* leerer Baum -> Abbruch */
   return ptr;


 if (isEqual(w, ptr->word) )              /* w ist wurzel -> baum vernichten */
   return destroyTree(ptr);


 if ( !isEmpty(NULL, ptr->left) && 
       isEqual(ptr->left->word, w) )      /* linker baum ist w -> diesen freigeben */
   {
      ptr->left=destroyTree(ptr->left);
      return ptr;
   }

 if ( !isEmpty(NULL, ptr->right) &&
       isEqual(ptr->right->word, w) )     /* rechter baum ist w -> diesen freigeben */
   {
      ptr->right=destroyTree(ptr->right);
      return ptr;
   }

 removePartTree(w, ptr->left);            /* links weitersuchen */

 removePartTree(w, ptr->right);           /* rechts weitersuchen */

 return ptr;
}


/******************************************************************************
   Zerstoert den Baum.
   @param Treenode*: Der Baum der zerstoert werden soll.
                     Handelt es sich bereits um einen leeren Baum,
		     so wird dieser zurueckgegeben.
   return Treenode*: ein leerer Baum
 ******************************************************************************/
Treenode* destroyTree(Treenode* ptr)
{
 if (isEmpty(NULL, ptr) )
   return mkEmpty();
 
 if (!isEmpty(NULL, ptr->left) )          /* lsche linken Teilbaum */
   destroyTree(ptr->left);
 
 if (!isEmpty(NULL, ptr->right) )         /* lsche rechten Teilbaum */
   destroyTree(ptr->right);
 
 if (ptr->word)                           /* lsche Wurzel */
   d_free(ptr->word);
 d_free(ptr);   

 return mkEmpty();
}

/******************************************************************************
Funktion sort:
@param1: Funktion f mit dem Parameter Treenode* und dem Rueckgabewert Treenode*.
@param2: Treenode* ptr
return: Treenode*
******************************************************************************/
Treenode* sort( Treenode* (*fkt) (Treenode*), Treenode *ptr)
{
 return (*fkt) (ptr);
}


/******************************************************************************
Funktion change:
@param1: Funktion f mit den Parametern (char* und Treenode*) und dem Rueckgabewert Treenode*.
@param2: char* w
@param3: Treenode* ptr
return: Treenode*
 ******************************************************************************/
Treenode* change( Treenode* (*fkt) (char *, Treenode *), char *w, Treenode *ptr)
{
 return (*fkt) (w, ptr);
}

/******************************************************************************
Funktion prad:
@param1: Funktion f mit den Parametern (char* und Treenode*) und dem Rueckgabewert int.
@param2: char* w
@param3: Treenode* ptr
return: int
 ******************************************************************************/
int prad( int (*fkt) (char *, Treenode *), char *w, Treenode *ptr)
{
 return (*fkt) (w, ptr);
}















