#include <stdio.h>

#include "btree.h"


Treenode *t1, *t2;


/******************************************************************************
 * Debug-Ausgabe-Routinen
 ******************************************************************************/


/*** gibt Fehler aus und beendet das Programm ***/
void printerror(int num)
{
 printf("\n---> Test %2d FEHLER!\n", num);
 exit(1); 
}


/*** gibt Test-okay aus ***/
void printokay(int num)
{
 printf("\n---> Test %2d erfolgreich!\n", num);
}



/******************************************************************************
 * TEST-FUNKTIONEN
 ******************************************************************************/

/************************************************/
/************************************************/
int test01(void)
{
 printf("\n1. --------- mkEmpty ---------\n");
 t1 = mkEmpty();
 printf("leerer Baum:\n");
 output(t1);
 return t1 == NULL;
}


/************************************************/
/************************************************/
int test02(void)
{
 int res=1;

 printf("\n2. --------- isEmpty ---------\n");
 t1 = mkEmpty();
 printf("leerer Baum:\n");
 output(t1);
 if (!isEmpty(NULL, t1) )     res=0;

 t1 = mkOne("test");
 printf("voller Baum:\n");
 output(t1);
 if (isEmpty(NULL, t1) )      res=0;
 t1 = destroyTree(t1); 

 return res;    
}


/************************************************/
/************************************************/
int test03(void)
{
 int res=1;

 printf("\n3. --------- isIn ---------\n");
 t1 = mkEmpty();
 printf("Suche in leerem Baum nach 'hallo':\n");
 output(t1);
 if ( isIn("hallo", t1) )     res=0;

 t1 = mkOne("test");
 printf("Suche in Baum mit Wurzel nach 'hallo':\n");
 output(t1);
 if ( isIn("hallo", t1) )     res=0;
 
 t1 = append("zwei", t1);
 t1 = append("klaus", t1);
 t1 = append("hier", t1);
 t1 = append("LANGSAM", t1);
 printf("Suche in Baum nach 'hallo':\n");
 output(t1);
 if ( isIn("hallo",t1) )      res=0;


 printf("Suche in Baum nach 'hier':\n");
 output(t1);
 if ( !isIn("hier",t1) )     res=0;

 t1 = destroyTree(t1); 

 return res;    
}


/************************************************/
/************************************************/
int test04(void)
{
 int res=1;

 printf("\n4. --------- mkOne ---------\n");
 printf("Erstelle Baum mit Leerstring:\n");
 t1 = mkOne("");
 output(t1);
 if ( !isIn("", t1) )         res=0;
 t1 = destroyTree(t1); 

 printf("Erstelle Baum mit keinem String:\n");
 t1 = mkOne(NULL);
 output(t1);
 if ( !isEmpty(NULL, t1) )    res=0;

 printf("Erstelle Baum mit 'Hallo' String:\n");
 t1 = mkOne("Hallo");
 output(t1);
 if ( !isIn("haLLo", t1) )    res=0;
 t1 = destroyTree(t1); 

 return res;    
}


/************************************************/
/************************************************/
int test05(void)
{
 int res=1;

 printf("\n5. --------- sortToRight ---------\n");
 printf("Erstelle Baum mit Leerstring:\n");
 t1 = mkOne("");
 output(t1);
 t1 = sortToRight(t1);
 printf("Baum nach Sortieren:\n");
 output(t1);
 if ( !isIn("", t1) )         res=0;
 t1 = destroyTree(t1); 

 printf("Erstelle Baum und sortiere ihn:\n");
 t1 = mkOne("Hallo");
 t1 = append("Anton", t1);
 t1 = append("Moritz", t1);
 t1 = append("anna", t1);
 t1 = append("Michaela", t1);
 t1 = append("shara", t1);
 t1 = append("TONI", t1);
 output(t1);
 t1 = sortToRight(t1);
 printf("Baum nach Sortieren:\n");
 output(t1);
 if (card(NULL, t1) != 7)     res=0;
 t1 = destroyTree(t1); 

 return res;    
}


/************************************************/
/************************************************/
int test06(void)
{
 int res=1;

 printf("\n6. --------- sortToLeft ---------\n");
 printf("Erstelle Baum mit Leerstring:\n");
 t1 = mkOne("");
 output(t1);
 t1 = sortToLeft(t1);
 printf("Baum nach Sortieren:\n");
 output(t1);
 if ( !isIn("", t1) )         res=0;
 t1 = destroyTree(t1); 

 printf("Erstelle Baum und sortiere ihn:\n");
 t1 = mkOne("Hallo");
 t1 = append("Anton", t1);
 t1 = append("Moritz", t1);
 t1 = append("anna", t1);
 t1 = append("Michaela", t1);
 t1 = append("shara", t1);
 t1 = append("TONI", t1);
 output(t1);
 t1 = sortToLeft(t1);
 printf("Baum nach Sortieren:\n");
 output(t1);

 printf("noch ein paar Mal hin und her:\n");
 t1 = sortToLeft(t1);
 t1 = sortToRight(t1);
 t1 = sortToLeft(t1);
 t1 = sortToRight(t1);
 t1 = sortToRight(t1);
 t1 = sortToRight(t1);
 t1 = sortToLeft(t1);
 printf("Baum nach hin und her:\n");
 output(t1);

 if (card(NULL, t1) != 7)     res=0;
 t1 = destroyTree(t1); 

 return res;    
}


/************************************************/
/************************************************/
int test07(void)
{
 int res=1;

 printf("\n7. --------- sortToBalance ---------\n");
 printf("Erstelle Baum mit Leerstring:\n");
 t1 = mkOne("");
 output(t1);
 t1 = sortToBalance(t1);
 printf("Baum nach Sortieren:\n");
 output(t1);
 if ( !isIn("", t1) )         res=0;
 t1 = destroyTree(t1); 

 printf("Erstelle Baum und sortiere ihn:\n");
 t1 = mkOne("Hallo");
 t1 = append("Anton", t1);
 t1 = append("Moritz", t1);
 t1 = append("anna", t1);
 t1 = append("Michaela", t1);
 t1 = append("shara", t1);
 t1 = append("TONI", t1);
 output(t1);
 t1 = sortToBalance(t1);
 printf("Baum nach Sortieren:\n");
 output(t1);

 printf("noch ein paar Mal hin und her:\n");
 t1 = sortToLeft(t1);
 t1 = sortToRight(t1);
 t1 = sortToLeft(t1);
 t1 = sortToRight(t1);
 t1 = sortToRight(t1);
 t1 = sortToRight(t1);
 t1 = sortToBalance(t1);
 printf("Baum nach hin und her:\n");
 output(t1);

 if (card(NULL, t1) != 7)     res=0;
 t1 = destroyTree(t1); 

 return res;    
}



/************************************************/
/************************************************/
int test08(void)
{
 int res=1;

 printf("\n8. --------- card ---------\n");
 printf("Erstelle Baum mit Leerstring:\n");
 t1 = mkOne("");
 output(t1);
 if ( card(NULL, t1) != 1)       res=0;
 t1 = destroyTree(t1); 

 printf("Erstelle langen Baum:\n");
 t1 = mkOne("Hallo");
 t1 = append("Anton", t1);
 t1 = append("Moritz", t1);
 t1 = append("anna", t1);
 t1 = append("Michaela", t1);
 t1 = append("shara", t1);
 t1 = append("TONI", t1);
 output(t1);
 if (card(NULL, t1) != 7)        res=0;

 printf("die gleichen elemente nochmal:\n");
 t1 = append("Anton", t1);
 t1 = append("Moritz", t1);
 t1 = append("anna", t1);
 t1 = append("Michaela", t1);
 t1 = append("shara", t1);
 t1 = append("TONI", t1);
 if (card(NULL, t1) != 7)        res=0;

 printf("und zwei dazu:\n");
 t1 = append("einer dazu", t1);
 t1 = append("und noch einer", t1);
 output(t1);
 if (card(NULL, t1) != 9)        res=0;

 t1 = destroyTree(t1); 
 if (card(NULL, t1) != 0)        res=0;

 return res;    
}


/************************************************/
/************************************************/
int test09(void)
{
 int res=1;

 printf("\n9. --------- ord ---------\n");
 printf("Erstelle Baum mit Leerstring:\n");
 t1 = mkOne("");
 output(t1);
 if ( ord("", t1) != 1)             res=0;
 t1 = destroyTree(t1); 

 printf("Erstelle langen Baum:\n");
 t1 = mkOne("Hallo");
 t1 = append("Anton", t1);
 t1 = append("Moritz", t1);
 t1 = append("anna", t1);
 t1 = append("Michaela", t1);
 t1 = append("shara", t1);
 t1 = append("TONI", t1);
 output(t1);
 if (ord(NULL, t1)      != 0)       res=0;
 if (ord("HaLLo",t1)    != 1)       res=0;
 if (ord("Anton",t1)    != 2)       res=0;
 if (ord("MoriTz",t1)   != 2)       res=0;
 if (ord("anna",t1)     != 3)       res=0;
 if (ord("Michaela",t1) != 3)       res=0;
 if (ord("shAra",t1)    != 3)       res=0;
 if (ord("TOnI",t1)     != 4)       res=0;

 t1 = destroyTree(t1); 
 if (ord("hallo",t1)     != 0)      res=0;

 return res;    
}


/************************************************/
/************************************************/
int test10(void)
{
 int res=1;

 printf("\n10. --------- append ---------\n");
 printf("Einfgen an Leerbaum:\n");
 t1 = mkEmpty();
 t1 = append("hallo", t1);
 output(t1);
 if (isEmpty(NULL, t1) )            res=0;

 printf("Einfgen an Wurzelbaum:\n");
 t1 = append("zwei", t1);
 output(t1);
 if ( card(NULL, t1) != 2)          res=0;

 printf("kompletten Baum zusammenbauen:\n");
 t1 = append("homer", t1);
 t1 = append("bender", t1);
 t1 = append("bart", t1);
 t1 = append("marge", t1);
 t1 = append("cartman", t1);
 t1 = append("bush", t1);
 t1 = append("clinton", t1);
 t1 = append("sieben", t1);
 t1 = append("hier", t1);
 t1 = append("telefon", t1);
 output(t1);
 if ( card(NULL, t1) != 12)         res=0;

 printf("Element einfgen, das bereits vorhanden ist:\n");
 t1 = append("sieben", t1);
 t1 = append("hier", t1);
 t1 = append("telefon", t1);
 if ( card(NULL, t1) != 12)         res=0;

 t1 = destroyTree(t1); 

 printf("Null einfgen:\n");
 t1 = append(NULL, t1);
 output(t1);
 if ( card(NULL, t1) != 0)          res=0;

 return res;    
}


/************************************************/
/************************************************/
int test11(void)
{
 int res=1;

 printf("\n11. --------- output ---------\n");
 printf("leeren Baum anzeigen:\n");
 t1 = mkEmpty();
 output(t1);

 printf("Wurzelbaum anzeigen:\n");
 t1 = mkOne("ein Element als Wurzel ... juhuuuuuuuu");
 output(t1); 

 printf("kompletten Baum anzeigen:\n");
 t1 = append("homer", t1);
 t1 = append("bender", t1);
 t1 = append("bart", t1);
 t1 = append("marge", t1);
 t1 = append("cartman", t1);
 t1 = append("bush", t1);
 t1 = append("clinton", t1);
 t1 = append("sieben", t1);
 output(t1); 
 t1 = destroyTree(t1);
 
 return res;    
}


/************************************************/
/************************************************/
int test12(void)
{
 int res=1;

 printf("\n12. --------- removeTree ---------\n");
 printf("leer Baum, Versuch Element zu entfernen:\n");
 t1 = mkEmpty();
 t1 = removeTree("", t1);
 output(t1);
 if (!isEmpty(NULL, t1))            res=0;

 printf("Nicht vorhandenes Element entfernen:\n");
 t1 = append("marko", t1);
 t1 = append("fritz", t1);
 t1 = append("peter", t1);
 t1 = append("steffi", t1);
 t1 = append("sandra", t1);
 t1 = append("marie", t1);
 t1 = append("uwe", t1);
 t1 = removeTree("nicht drin", t1);
 if ( card(NULL, t1) != 7)          res=0;
 output(t1);

 printf("Element (marie) ohne Nachfolger entfernen:\n");
 t1 = removeTree("marie", t1);
 if ( card(NULL, t1) != 6)          res=0;
 output(t1);

 printf("Element (peter) mit Nachfolger entfernen:\n");
 t1 = removeTree("peter", t1);
 if ( card(NULL, t1) != 5)          res=0;
 output(t1);

 printf("Element (steffi) mit Nachfolger entfernen:\n");
 t1 = removeTree("STeffi", t1);
 if ( card(NULL, t1) != 4)          res=0;
 output(t1);

 printf("Wurzel (marko) mit Nachfolger entfernen:\n");
 t1 = removeTree("MARKO", t1);
 if ( card(NULL, t1) != 3)          res=0;
 output(t1);

 printf("Element (sandra) mit Nachfolger entfernen:\n");
 t1 = removeTree("Sandra", t1);
 if ( card(NULL, t1) != 2)          res=0;
 output(t1);

 printf("Element (uwe) mit Nachfolger entfernen:\n");
 t1 = removeTree("Uwe", t1);
 if ( card(NULL, t1) != 1)          res=0;
 output(t1);

 printf("Letztes Element (fritz) entfernen:\n");
 t1 = removeTree("frItz", t1);
 if ( card(NULL, t1) != 0)          res=0;
 output(t1);

 return res;    
}


/************************************************/
/************************************************/
int test13(void)
{
 int res=1;

 printf("\n13. --------- removePartTree ---------\n");
 printf("leer Baum, Versuch Element zu entfernen:\n");
 t1 = mkEmpty();
 t1 = removePartTree("hallo", t1);
 if (!isEmpty(NULL, t1) )           res=0;

 printf("Nicht vorhandenes Element entfernen:\n");
 t1 = append("eins", t1);
 t1 = append("zwei", t1);
 t1 = append("drei", t1);
 output(t1);
 t1 = removePartTree("nicht drin", t1);
 if (card(NULL, t1) != 3)           res=0;
 printf("nach remove Part (nicht drin):\n");
 output(t1);

 printf("Element ohne Nachfolger entfernen:\n");
 t1 = removePartTree("zwei", t1);
 if (card(NULL, t1) != 2)           res=0;
 printf("nach remove Part (zwei):\n");
 output(t1);

 printf("kleinen Teilbaum entfernen\n");
 t1 = append("homer", t1);
 t1 = append("bender", t1);
 t1 = append("bart", t1);
 t1 = append("marge", t1);
 t1 = append("cartman", t1);
 t1 = append("bush", t1);
 t1 = append("clinton", t1);
 t1 = append("sieben", t1);
 output(t1);
 t1 = removePartTree("Homer", t1);
 printf("nach remove Part (homer):\n");
 output(t1);

 printf("Wurzel entfernen:\n");
 t1 = removePartTree("eins", t1);
 printf("nach remove Part (eins):\n");
 output(t1);
 if (card(NULL, t1) != 0)           res=0;

 return res;    
}


/************************************************/
/************************************************/
int test14(void)
{
 int res=1;

 printf("\n14. --------- destroyTree ---------\n");
 printf("leer Baum entfernen:\n");
 t1 = mkEmpty();
 t1 = destroyTree(t1);
 printf("nach destroy:\n");
 output(t1);
 if (!isEmpty(NULL, t1) )           res=0;

 printf("Wurzelbaum entfernen:\n");
 t1 = append("wurzelbaum", t1);
 output(t1);
 t1 = destroyTree(t1);
 printf("nach destroy:\n");
 output(t1);
 if (!isEmpty(NULL, t1) )           res=0;

 printf("kompletten Baum entfernen:\n");
 t1 = append("homer", t1);
 t1 = append("bender", t1);
 t1 = append("bart", t1);
 t1 = append("marge", t1);
 t1 = append("cartman", t1);
 t1 = append("bush", t1);
 t1 = append("clinton", t1);
 t1 = append("sieben", t1);
 output(t1);
 t1 = destroyTree(t1);
 printf("nach destroy:\n");
 output(t1);
 if (!isEmpty(NULL, t1) )           res=0;

 return res;    
}


/************************************************/
/************************************************/
int testfktptr(void)
{
 int res=1;

 printf("\n15. --------- Funktionszeiger testen ---------\n");

 t1 = mkEmpty();

 if (prad(isEmpty, NULL, t1) != 1)  res=0;
 printf("isEmpty: %s\n", res ? "okay" : "Fehler!");

 t1 = mkOne("neu");

 if (prad(isIn, "neu", t1) != 1)    res=0;
 printf("isIn: %s\n", res ? "okay" : "Fehler!");

 t1 = change(append, "hallo", t1);   
 t1 = change(append, "nase", t1);   
 t1 = change(append, "zwerg", t1);   
 
 if (prad(card, NULL, t1) != 4)     res=0;
 printf("append und card: %s\n", res ? "okay" : "Fehler!");
 output(t1);
 printf("\nSort To Left:\n");
 t1 = sort(sortToLeft, t1);
 output(t1);
 printf("\nSort To Right:\n");
 t1 = sort(sortToRight, t1);
 output(t1);
 printf("\nSort To Balance:\n");
 t1 = sort(sortToBalance, t1);
 output(t1);

 if (prad(ord, "hallo", t1) != 3)   res=0;
 printf("ord: %s\n", res ? "okay" : "Fehler!");

 t1 = change(removeTree, "neu", t1);
 if (prad(card, NULL, t1) != 3)     res=0;
 printf("removeTree von neu: %s\n", res ? "okay" : "Fehler!");
 output(t1);

 t1 = change(removePartTree, "hallo", t1);
 if (prad(card, NULL, t1) != 2)     res=0;
 printf("removePartTree von hallo: %s\n", res ? "okay" : "Fehler!");
 output(t1);

 t1 = sort(destroyTree, t1);
 if (prad(isEmpty, NULL, t1) != 1 ) res=0;
 printf("destroytree: %s\n", res ? "okay" : "Fehler!");

 return res;    
}




/******************************************************************************
 * Hauptprogramm zum Testen der Lib
 ******************************************************************************/
int main (int argc, char * argv[])
{

 if (test01() )   printokay(1);   else             printerror(1);

 if (test02() )   printokay(2);   else             printerror(2);

 if (test03() )   printokay(3);   else             printerror(3);

 if (test04() )   printokay(4);   else             printerror(4);

 if (test05() )   printokay(5);   else             printerror(5);

 if (test06() )   printokay(6);   else             printerror(6);

 if (test07() )   printokay(7);   else             printerror(7);

 if (test08() )   printokay(8);   else             printerror(8);

 if (test09() )   printokay(9);   else             printerror(9);

 if (test10() )   printokay(10);  else             printerror(10);

 if (test11() )   printokay(11);  else             printerror(11);

 if (test12() )   printokay(12);  else             printerror(12);

 if (test13() )   printokay(13);  else             printerror(13);

 if (test14() )   printokay(14);  else             printerror(14);
/*
 if (testfktptr() )   printokay(15);  else         printerror(15);
 */
 printf("\nmemcount: %d\n", getmemcount() );
 
 if ( getmemcount() )
   printf("\n\n--> !!! ACHTUNG TEST FEHLGESCHLAGEN !!! <--\n\n");
 else
   printf("\n\n--> TEST ERFOLGREICH!\n\n");


 return 0;
}
 
